using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;

namespace Ato.EN.IntegrationServices.CodeGenerationFITR
{
    public class FITR2021Consumer
    {

        private static Regex StripWhitespace = new Regex(@"\s*");

        #region Helper/Validation Methods
        private static bool? ToBoolean(string str)
        {
            bool returnValue;
            string value = (str ?? "").Trim();
            if (value == "0" || value == "1")
                return Convert.ToBoolean(Convert.ToInt32(value));

            if (Boolean.TryParse(value, out returnValue))
                return returnValue;
            return null;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Derives the SBR element location.
        /// </summary>
        /// <param name="sbrElement">The SBR element.</param>
        /// <param name="addOccurrenceIndex">if set to <c>true</c> [add occurrence the occurrence index to the path].</param>
        /// <returns></returns>
        private static string DeriveSbrElementLocation(SBRElement sbrElement, bool addOccurrenceIndex = false)
        {
            if (sbrElement == null) return "/xbrli:xbrl";
            int xpathOccurrence = sbrElement.OccurrenceIndex + 1;
            string location;

            if (addOccurrenceIndex)
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name + "[" +
                           xpathOccurrence + "]";
            }
            else
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name;
            }

            if (sbrElement.IsInTuple)
            {
                location = DeriveSbrElementLocation(sbrElement.ParentTuple, true) + location;
            }
            else
            {
                location = "/xbrli:xbrl" + location;
            }

            return location;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        #endregion

        public FITR2021 Consume(SBRDocument sbrDocument)
        {
            List<ProcessMessageDocument> errors;
            return Consume(sbrDocument, false, out errors);
        }

        /// <summary>
        /// Allows for validation of rules that can not be generated and must run at a lower level.
        /// A sub-class will need to be created with an override for this method - which does nothing in its base implementation.
        /// </summary>
        /// <param name="sbrDocument">The SBR document.</param>
        /// <param name="populatedReportElementsMap">The populated report elements map can be used to find data elements and tuples.</param>
        /// <param name="contextArray">The context array can be used to find context values.</param>
        /// <param name="errors">The errors collection is to be populated with validation results.</param>
        protected virtual void ValidateUnsupportedRules(SBRDocument sbrDocument, IDictionary<string, SBRElement> populatedReportElementsMap, SBRContext[] contextArray, List<ProcessMessageDocument> errors)
        {

        }

        public FITR2021 Consume(SBRDocument sbrDocument, bool validationMode, out List<ProcessMessageDocument> errors)
        {
            FITR2021 report;
            report = new FITR2021();
            errors = new List<ProcessMessageDocument>();

            SBRContext[] contextArray;
            IDictionary<string, SBRElement> populatedReportElementsMap;

            contextArray = sbrDocument.ContextsMap.Values.ToArray();
            populatedReportElementsMap = sbrDocument.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

            if (validationMode) ValidateUnsupportedRules(sbrDocument, populatedReportElementsMap, contextArray, errors);

            /// <summary>
            /// Current element being extracted
            /// </summary>
            SBRElement currentValue = null;
            /// <summary>
            /// Current elements being extracted
            /// </summary>
            IList<SBRElement> currentValues = null;
            /// <summary>
            /// Current occurrences being extracted
            /// </summary>
            IList<SBRElement> currentOccurrences = null;
            /// <summary>
            /// Working Variable for date time variables
            /// </summary>
            DateTime currentDateTimeValue;
            /// <summary>
            /// Working Variable for boolean variables
            /// </summary>
            bool currentBooleanValue;
            /// <summary>
            /// Working Variable for decimal variables
            /// </summary>
            decimal currentDecimalValue;
            /// <summary>
            /// Working Variable for integer variables
            /// </summary>
            int currentIntValue;
            /// <summary>
            /// Working Variable for long variables
            /// </summary>
            long currentLongValue;
            XmlDocumentFragment currentSegment;
            XmlNamespaceManager currentNamespaceManager;
            XmlNode currentNode;

            #region sbrINTContext
            IEnumerable<SBRContext> sbrINTContexts = null;
            SBRContext sbrINTContext = null;

            sbrINTContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Intermediary")))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedINTElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrINTContexts != null && sbrINTContexts.Count() > 0)
            {
                sbrINTContext = sbrINTContexts.First();

                report.INTId = sbrINTContext.Id;
                report.INTIdentifier = sbrINTContext.EntityIdentifier;
                report.INTIdentifierScheme = sbrINTContext.EntityScheme;

                report.INTCount = sbrINTContexts.Count();
                report.INTExists = true;
                report.INTOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContext) + 1;
                report.INTLastOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContexts.Last()) + 1;


                if (report.INTIdentifierScheme == "http://www.ato.gov.au/abn") report.INTIdentifierABN = report.INTIdentifier;

                report.INTStartDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationStartAsDateObject;
                report.INTEndDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationEndAsDateObject;

                sbrPopulatedINTElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrINTContexts.Count(); i++)
                {
                    sbrINTContext = sbrINTContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrINTContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedINTElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedINTElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedINTElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    IEnumerable<SBRElement> sbrDeclarationCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration", out currentValue))
                        sbrDeclarationCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationCollection = null;

                    #region sbrDeclarationCollection

                    if (sbrDeclarationCollection != null)
                    {
                        report.DeclarationCollectionExists = true;
                        report.DeclarationCollectionCount = sbrDeclarationCollection.Count();

                        IEnumerable<SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTCollection;
                        sbrDeclarationCollection_TrueAndCorrect_INTCollection = sbrDeclarationCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrDeclarationCollection_TrueAndCorrect_INTCollection != null && sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count() > 0)
                        {
                            SBRElement sbrDeclarationCollection_TrueAndCorrect_INT;
                            sbrDeclarationCollection_TrueAndCorrect_INT = sbrDeclarationCollection_TrueAndCorrect_INTCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap;
                            sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap = sbrDeclarationCollection_TrueAndCorrect_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.DeclarationCollection_TrueAndCorrect_INTExists = true;
                            report.DeclarationCollection_TrueAndCorrect_INTCount = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count();
                            report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INT.OccurrenceIndex + 1;
                            report.DeclarationCollection_TrueAndCorrect_INTLastOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Last().OccurrenceIndex + 1;

                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR164 = currentValue.Value;
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementAccepted.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR165 = ToBoolean(currentValue.Value);
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Statement.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR124 = currentValue.Value;
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Signature.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR125 = DateTime.Parse(currentValue.Value);
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.SignatoryIdentifier.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR163 = currentValue.Value;
                            }

                            IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection;
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName", out currentValue))
                                sbrDeclaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                            else
                                sbrDeclaration_PersonUnstructuredNameCollection = null;

                            #region sbrDeclaration_PersonUnstructuredNameCollection

                            if (sbrDeclaration_PersonUnstructuredNameCollection != null)
                            {
                                report.Declaration_PersonUnstructuredNameCollectionExists = true;
                                report.Declaration_PersonUnstructuredNameCollectionCount = sbrDeclaration_PersonUnstructuredNameCollection.Count();

                                IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection;
                                sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection = sbrDeclaration_PersonUnstructuredNameCollection.Where(
                                    ce => ce.ChildElements != null &&
                                    ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                    sbrINTContexts.Contains(te.Context)));
                                if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection != null && sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count() > 0)
                                {
                                    SBRElement sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.ElementAt(0);

                                    IDictionary<string, SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists = true;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCount = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count();
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.OccurrenceIndex + 1;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTLastOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Last().OccurrenceIndex + 1;

                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.Usage.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FITR168 = currentValue.Value;
                                    }

                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.FullName.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FITR169 = currentValue.Value;
                                    }
                                }
                            }
                            #endregion End of Tuple sbrDeclaration_PersonUnstructuredNameCollection
                            if (validationMode) ValidateINTDeclaration_PersonUnstructuredNameCollection(errors, sbrDeclaration_PersonUnstructuredNameCollection, sbrINTContexts, report);

                        }
                    }
                    #endregion End of Tuple sbrDeclarationCollection
                    if (validationMode) ValidateINTDeclarationCollection(errors, sbrDeclarationCollection, sbrINTContexts, report);


                    IEnumerable<SBRElement> sbrPersonNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails", out currentValue))
                        sbrPersonNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrPersonNameDetailsCollection = null;

                    #region sbrPersonNameDetailsCollection

                    if (sbrPersonNameDetailsCollection != null)
                    {
                        report.PersonNameDetailsCollectionExists = true;
                        report.PersonNameDetailsCollectionCount = sbrPersonNameDetailsCollection.Count();

                        IEnumerable<SBRElement> sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection;
                        sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection = sbrPersonNameDetailsCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Contact") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection != null && sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection.Count() > 0)
                        {
                            SBRElement sbrPersonNameDetailsCollection_LGL_Contact_C_INT;
                            sbrPersonNameDetailsCollection_LGL_Contact_C_INT = sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap;
                            sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap = sbrPersonNameDetailsCollection_LGL_Contact_C_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.PersonNameDetailsCollection_LGL_Contact_C_INTExists = true;
                            report.PersonNameDetailsCollection_LGL_Contact_C_INTCount = sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection.Count();
                            report.PersonNameDetailsCollection_LGL_Contact_C_INTOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_Contact_C_INT.OccurrenceIndex + 1;
                            report.PersonNameDetailsCollection_LGL_Contact_C_INTLastOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_Contact_C_INTCollection.Last().OccurrenceIndex + 1;

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.PersonNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR153 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR155 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR154 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Title.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR111 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.NameSuffix.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR113 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.FamilyName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR112 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.GivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR114 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.OtherGivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR115 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrPersonNameDetailsCollection
                    if (validationMode) ValidateINTPersonNameDetailsCollection(errors, sbrPersonNameDetailsCollection, sbrINTContexts, report);


                    IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails", out currentValue))
                        sbrOrganisationNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrOrganisationNameDetailsCollection = null;

                    #region sbrOrganisationNameDetailsCollection

                    if (sbrOrganisationNameDetailsCollection != null)
                    {
                        report.OrganisationNameDetailsCollectionExists = true;
                        report.OrganisationNameDetailsCollectionCount = sbrOrganisationNameDetailsCollection.Count();

                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_C_INTCollection;
                        sbrOrganisationNameDetailsCollection_MN_C_INTCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_C_INTCollection != null && sbrOrganisationNameDetailsCollection_MN_C_INTCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_C_INT;
                            sbrOrganisationNameDetailsCollection_MN_C_INT = sbrOrganisationNameDetailsCollection_MN_C_INTCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_C_INTTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_C_INTTupleElementMap = sbrOrganisationNameDetailsCollection_MN_C_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.OrganisationNameDetailsCollection_MN_C_INTExists = true;
                            report.OrganisationNameDetailsCollection_MN_C_INTCount = sbrOrganisationNameDetailsCollection_MN_C_INTCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_C_INTOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_INT.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_C_INTLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_INTCollection.Last().OccurrenceIndex + 1;

                            if (sbrOrganisationNameDetailsCollection_MN_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR159 = currentValue.Value;
                            }

                            if (sbrOrganisationNameDetailsCollection_MN_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR198 = currentValue.Value;
                            }

                            if (sbrOrganisationNameDetailsCollection_MN_C_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR116 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateINTOrganisationNameDetailsCollection(errors, sbrOrganisationNameDetailsCollection, sbrINTContexts, report);


                    IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContactTelephone", out currentValue))
                        sbrElectronicContactTelephoneCollection = currentValue.Occurrences;
                    else
                        sbrElectronicContactTelephoneCollection = null;

                    #region sbrElectronicContactTelephoneCollection

                    if (sbrElectronicContactTelephoneCollection != null)
                    {
                        report.ElectronicContactTelephoneCollectionExists = true;
                        report.ElectronicContactTelephoneCollectionCount = sbrElectronicContactTelephoneCollection.Count();

                        IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection_03_INTCollection;
                        sbrElectronicContactTelephoneCollection_03_INTCollection = sbrElectronicContactTelephoneCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "ElectronicContact.Telephone.Usage.Code" && te.Value == "03" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrElectronicContactTelephoneCollection_03_INTCollection != null && sbrElectronicContactTelephoneCollection_03_INTCollection.Count() > 0)
                        {
                            SBRElement sbrElectronicContactTelephoneCollection_03_INT;
                            sbrElectronicContactTelephoneCollection_03_INT = sbrElectronicContactTelephoneCollection_03_INTCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrElectronicContactTelephoneCollection_03_INTTupleElementMap;
                            sbrElectronicContactTelephoneCollection_03_INTTupleElementMap = sbrElectronicContactTelephoneCollection_03_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.ElectronicContactTelephoneCollection_03_INTExists = true;
                            report.ElectronicContactTelephoneCollection_03_INTCount = sbrElectronicContactTelephoneCollection_03_INTCollection.Count();
                            report.ElectronicContactTelephoneCollection_03_INTOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INT.OccurrenceIndex + 1;
                            report.ElectronicContactTelephoneCollection_03_INTLastOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INTCollection.Last().OccurrenceIndex + 1;

                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR156 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.ServiceLine.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR157 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Area.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR117 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Minimal.Number", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR118 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrElectronicContactTelephoneCollection
                    if (validationMode) ValidateINTElectronicContactTelephoneCollection(errors, sbrElectronicContactTelephoneCollection, sbrINTContexts, report);


                    if (sbrPopulatedINTElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Identifiers.TaxAgentClientReference.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR119 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Identifiers.TaxAgentNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR2 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrINTContext

            #region sbrRPContext
            IEnumerable<SBRContext> sbrRPContexts = null;
            SBRContext sbrRPContext = null;

            sbrRPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPContexts != null && sbrRPContexts.Count() > 0)
            {
                sbrRPContext = sbrRPContexts.First();

                report.RPId = sbrRPContext.Id;
                report.RPIdentifier = sbrRPContext.EntityIdentifier;
                report.RPIdentifierScheme = sbrRPContext.EntityScheme;

                report.RPCount = sbrRPContexts.Count();
                report.RPExists = true;
                report.RPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContext) + 1;
                report.RPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContexts.Last()) + 1;


                if (report.RPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPIdentifierTFN = report.RPIdentifier;

                report.RPStartDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationStartAsDateObject;
                report.RPEndDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPContexts.Count(); i++)
                {
                    sbrRPContext = sbrRPContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }
                    report.FITR5 = report.RPIdentifier;

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.TargetFinancial.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR1 = int.Parse(currentValues[0].Value);
                    }

                    IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails", out currentValue))
                        sbrOrganisationNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrOrganisationNameDetailsCollection = null;

                    #region sbrOrganisationNameDetailsCollection

                    if (sbrOrganisationNameDetailsCollection != null)
                    {

                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPCollection;
                        sbrOrganisationNameDetailsCollection_MN_C_RPCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_C_RP;
                            sbrOrganisationNameDetailsCollection_MN_C_RP = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrOrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.OrganisationNameDetailsCollection_MN_C_RPExists = true;
                            report.OrganisationNameDetailsCollection_MN_C_RPCount = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR151 = currentValue.Value;
                            }

                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR192 = currentValue.Value;
                            }

                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR6 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateRPOrganisationNameDetailsCollection(errors, sbrOrganisationNameDetailsCollection, sbrRPContexts, report);


                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR7 = currentValues[0].Value;
                    }

                    IEnumerable<SBRElement> sbrAddressDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails", out currentValue))
                        sbrAddressDetailsCollection = currentValue.Occurrences;
                    else
                        sbrAddressDetailsCollection = null;

                    #region sbrAddressDetailsCollection

                    if (sbrAddressDetailsCollection != null)
                    {
                        report.AddressDetailsCollectionExists = true;
                        report.AddressDetailsCollectionCount = sbrAddressDetailsCollection.Count();

                        IEnumerable<SBRElement> sbrAddressDetailsCollection_POS_C_RPCollection;
                        sbrAddressDetailsCollection_POS_C_RPCollection = sbrAddressDetailsCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrAddressDetailsCollection_POS_C_RPCollection != null && sbrAddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrAddressDetailsCollection_POS_C_RP;
                            sbrAddressDetailsCollection_POS_C_RP = sbrAddressDetailsCollection_POS_C_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrAddressDetailsCollection_POS_C_RPTupleElementMap;
                            sbrAddressDetailsCollection_POS_C_RPTupleElementMap = sbrAddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.AddressDetailsCollection_POS_C_RPExists = true;
                            report.AddressDetailsCollection_POS_C_RPCount = sbrAddressDetailsCollection_POS_C_RPCollection.Count();
                            report.AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                            report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.OverseasAddress.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR143 = ToBoolean(currentValue.Value);
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR144 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR145 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line1.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR8 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line2.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR146 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line3.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR147 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line4.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR148 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.LocalityName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR9 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Postcode.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR11 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.StateOrTerritory.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR10 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.CountryName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR149 = currentValue.Value;
                            }

                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Country.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR150 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrAddressDetailsCollection
                    if (validationMode) ValidateRPAddressDetailsCollection(errors, sbrAddressDetailsCollection, sbrRPContexts, report);


                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.Amendment.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR171 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.AmendmentSequence.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR174 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.AmendmentReason.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR173 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.AmendmentType.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR172 = currentValues[0].Value;
                    }

                    IEnumerable<SBRElement> sbrTrusteeCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Trustee", out currentValue))
                        sbrTrusteeCollection = currentValue.Occurrences;
                    else
                        sbrTrusteeCollection = null;

                    #region sbrTrusteeCollection

                    if (sbrTrusteeCollection != null)
                    {
                        report.TrusteeCollectionExists = true;
                        report.TrusteeCollectionCount = sbrTrusteeCollection.Count();
                        SBRElement sbrTrustee;
                        sbrTrustee = sbrTrusteeCollection.ElementAt(0);

                        IDictionary<string, SBRElement> sbrTrusteeTupleElementMap;
                        sbrTrusteeTupleElementMap = sbrTrustee.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                        IEnumerable<SBRElement> sbrTrustee_OrganisationNameDetailsCollection;
                        if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails", out currentValue))
                            sbrTrustee_OrganisationNameDetailsCollection = currentValue.Occurrences;
                        else
                            sbrTrustee_OrganisationNameDetailsCollection = null;

                        #region sbrTrustee_OrganisationNameDetailsCollection

                        if (sbrTrustee_OrganisationNameDetailsCollection != null)
                        {
                            report.Trustee_OrganisationNameDetailsCollectionExists = true;
                            report.Trustee_OrganisationNameDetailsCollectionCount = sbrTrustee_OrganisationNameDetailsCollection.Count();

                            IEnumerable<SBRElement> sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection;
                            sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrTrustee_OrganisationNameDetailsCollection.Where(
                                ce => ce.ChildElements != null &&
                                ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP;
                                sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);

                                IDictionary<string, SBRElement> sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                                report.Trustee_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                report.Trustee_OrganisationNameDetailsCollection_MN_C_RPCount = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                report.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                report.Trustee_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;

                                if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FITR152 = currentValue.Value;
                                }

                                if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FITR194 = currentValue.Value;
                                }

                                if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FITR12 = currentValue.Value;
                                }
                            }
                        }
                        #endregion End of Tuple sbrTrustee_OrganisationNameDetailsCollection
                        if (validationMode) ValidateRPTrustee_OrganisationNameDetailsCollection(errors, sbrTrustee_OrganisationNameDetailsCollection, sbrRPContexts, report);


                        if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Identifiers.AustralianBusinessNumber.Identifier", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR13 = currentValue.Value;
                        }
                    }
                    #endregion End of Tuple sbrTrusteeCollection

                    IEnumerable<SBRElement> sbrFinancialInstitutionAccountCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount", out currentValue))
                        sbrFinancialInstitutionAccountCollection = currentValue.Occurrences;
                    else
                        sbrFinancialInstitutionAccountCollection = null;

                    #region sbrFinancialInstitutionAccountCollection

                    if (sbrFinancialInstitutionAccountCollection != null)
                    {
                        report.FinancialInstitutionAccountCollectionExists = true;
                        report.FinancialInstitutionAccountCollectionCount = sbrFinancialInstitutionAccountCollection.Count();
                        SBRElement sbrFinancialInstitutionAccount;
                        sbrFinancialInstitutionAccount = sbrFinancialInstitutionAccountCollection.ElementAt(0);

                        IDictionary<string, SBRElement> sbrFinancialInstitutionAccountTupleElementMap;
                        sbrFinancialInstitutionAccountTupleElementMap = sbrFinancialInstitutionAccount.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                        if (sbrFinancialInstitutionAccountTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount.BankStateBranch.Number", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR176 = currentValue.Value;
                        }

                        if (sbrFinancialInstitutionAccountTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount.FinancialInstitutionAccount.Number", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR177 = currentValue.Value;
                        }

                        if (sbrFinancialInstitutionAccountTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount.FinancialInstitutionAccountName.Text", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR178 = currentValue.Value;
                        }

                        if (sbrFinancialInstitutionAccountTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount.FinancialInstitutionBranchName.Text", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR179 = currentValue.Value;
                        }

                        if (sbrFinancialInstitutionAccountTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}FinancialInstitutionAccount.FinancialInstitutionName.Text", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FITR180 = currentValue.Value;
                        }
                    }
                    #endregion End of Tuple sbrFinancialInstitutionAccountCollection

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Party.Type.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR14 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationFundDetails.AustralianSuperannuationFundStatus.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR15 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationFundDetails.FundBenefitStructure.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR16 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationDetails.Establishment.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR18 = DateTime.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationDetails.OperationCease.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR19 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationDetails.OrganisationActivityStatusEnd.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR20 = DateTime.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationDetails.SignificantGlobalEntityStatus.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR212 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.CountryByCountryReportingEntityStatus.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR219 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}CapitalGainsTax.Event.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR21 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}CapitalGainsTax.ExemptionOrRolloverApplied.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR203 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}CapitalGainsTax.ExemptionOrRolloverApplied.Code", out currentValues))
                    {
                        report.FITR204Collection = new List<string>();
                        try
                        {
                            foreach (SBRElement occurrence in currentValues)
                            {
                                if (!occurrence.IsNil)
                                    report.FITR204Collection.Add(occurrence.Value);
                            }
                        }
                        catch (Exception)
                        {
                            // The above code is known to throw exceptions in some cases but not others.
                            // We don't really know why but think that catching them like this will result in correct behaviour
                        }
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.CapitalGainsNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR23 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.RentLeasingHiringGross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR24 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Interest.Gross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR25 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.ForestryManagedInvestmentScheme.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR26 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR29 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Equity.InternationalDealings.ForeignFundTransfers.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR30 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.ForeignFundTransfers.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR31 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Remuneration.ABNNotQuotedPaymentGross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR32 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.PartnershipDistributionGross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR33 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.DividendsUnfranked.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR34 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.DividendsFranked.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR35 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.FrankingCredits.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR36 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.TrustDistributionUnfranked.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR37 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.TrustDistributionFranked.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR38 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR39 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.TrustDistributionOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR40 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.EmployerAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR41 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.PersonalAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR42 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.EmployerContributionsTaxFileNumberNotQuoted.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR43 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.ContributionsExcludedByTrustee.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR44 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.ReducedContributions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR45 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationContribution.LiabilityTransferToLifeInsuranceCompanyOrPooledSuperannuationTrustAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR46 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.ForeignExchangeGains.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR186 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Other.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR47 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Other.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR48 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.AssessableIncomeDueToFundTaxStatusChange.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR49 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.NonArmsLengthOtherNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR50 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxConcession.PensionIncomeExemptDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR200 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Remuneration.WagesandSalaries.Payments.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR54 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.CapitalWorksDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR55 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.DepreciationAllowableDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR56 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.PremiumsDeathOrDisability.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR57 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.Investment.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR59 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.Operating.ManagementExpense.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR190 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.Operating.AdministrativeExpense.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR188 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.ForestryManagedInvestmentSchemeDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR61 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.ForeignExchangeLosses.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR187 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.DeductibleOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR62 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.DeductibleOther.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR63 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Deduction.TaxLossesDeducted.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR64 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Taxable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR65 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TotalTaxOnTaxableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR182 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TFNNotQuotedContributionsTax.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR181 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxableGross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR66 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.InternationalDealings.TaxOffset.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR67 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetsAndRebatesTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR68 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetNonRefundableNonCarryForward.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR189 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.SubtotalTaxPayable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR183 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetNonRefundableCarryForward.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR215 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetCarriedForwardSubTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR216 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.FrankingCredits.Refundable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR73 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetTFNNotQuoted.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR131 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetEntitlementForNationalRentalAffordabilityScheme.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR133 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetExplorationCredits.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR206 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TotalRefundableTaxOffsets.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR184 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Payable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR199 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.AAMInterestSection102.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR69 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Interest.EarlyPaymentCredit.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR70 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR71 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR72 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR175 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TaxOffsetCalculatedInterestTFNNotQuoted.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR132 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR209 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Deduction.CreditsEligibleTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR201 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.TotalRemainderOfRefundableTaxOffsets.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR185 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.LiabilityInstalmentsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR74 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.PayableOrRefundableTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR75 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Equity.NetAssetsCalculated.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR83 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Superannuation.TransferredAssessable.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR84 = ToBoolean(currentValues[0].Value);
                    }

                    IEnumerable<SBRElement> sbrTransfereeCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Transferee", out currentValue))
                        sbrTransfereeCollection = currentValue.Occurrences;
                    else
                        sbrTransfereeCollection = null;

                    #region sbrTransfereeCollection

                    if (sbrTransfereeCollection != null)
                    {
                        report.TransfereeCollectionExists = true;
                        report.TransfereeCollectionCount = sbrTransfereeCollection.Count();
                        if (sbrTransfereeCollection != null && sbrTransfereeCollection.Count() > 0)
                        {
                            report.TransfereeCollection = new List<FITR2021.Transferee>();
                            for (int tupleIndex = 0; tupleIndex < sbrTransfereeCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrTransferee;
                                sbrTransferee = sbrTransfereeCollection.ElementAt(tupleIndex);

                                IDictionary<string, SBRElement> sbrTransfereeTupleElementMap;
                                sbrTransfereeTupleElementMap = sbrTransferee.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                                FITR2021.Transferee transferee = new FITR2021.Transferee();
                                report.TransfereeCollection.Add(transferee);
                                transferee.OccurrenceIndex = tupleIndex + 1;

                                IEnumerable<SBRElement> sbrTransferee_OrganisationNameDetailsCollection;
                                if (sbrTransfereeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails", out currentValue))
                                    sbrTransferee_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTransferee_OrganisationNameDetailsCollection = null;

                                #region sbrTransferee_OrganisationNameDetailsCollection

                                if (sbrTransferee_OrganisationNameDetailsCollection != null)
                                {
                                    transferee.Transferee_OrganisationNameDetailsCollectionExists = true;
                                    transferee.Transferee_OrganisationNameDetailsCollectionCount = sbrTransferee_OrganisationNameDetailsCollection.Count();

                                    IEnumerable<SBRElement> sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrTransferee_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTransferee_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrTransferee_OrganisationNameDetailsCollection_MN_C_RP = sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);

                                        IDictionary<string, SBRElement> sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrTransferee_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                                        transferee.Transferee_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        transferee.Transferee_OrganisationNameDetailsCollection_MN_C_RPCount = sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        transferee.Transferee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrTransferee_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        transferee.Transferee_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;

                                        if (sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) transferee.FITR170 = currentValue.Value;
                                        }

                                        if (sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) transferee.FITR196 = currentValue.Value;
                                        }

                                        if (sbrTransferee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) transferee.FITR85 = currentValue.Value;
                                        }
                                    }
                                }
                                #endregion End of Tuple sbrTransferee_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPTransferee_OrganisationNameDetailsCollection(errors, sbrTransferee_OrganisationNameDetailsCollection, sbrRPContexts, report);


                                if (sbrTransfereeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Identifiers.AustralianBusinessNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) transferee.FITR86 = currentValue.Value;
                                }

                                if (sbrTransfereeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Superannuation.TransferredAssessable.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) transferee.FITR87 = decimal.Parse(currentValue.Value);
                                }

                                if (sbrTransfereeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Superannuation.MarketValueTransfer.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) transferee.FITR88 = decimal.Parse(currentValue.Value);
                                }
                            }
                            #endregion End of Tuple sbrTransfereeCollection
                        }
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Superannuation.MarketValueTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR89 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.RelatedPartiesTransactions.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR90 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR91 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.DirectOrIndirectOverseasInterest.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR92 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.ForeignSourceIncomeExcessAggregateValue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR93 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}InternationalDealings.TransactionsWithTaxHavenCountries.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR94 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxConcession.ClaimedExemptCurrentPensionIncomeAndCertificateObtained.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR95 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Deduction.ClaimedDeathOrDisabilityDeductionWhichRequiresCertificate.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR96 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}IncomeTax.Deduction.ClaimedDeathOrDisabilityDeductionCertificateObtained.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR97 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}SuperannuationPayment.PaymentsToContributingEmployersAndAssociates.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR98 = ToBoolean(currentValues[0].Value);
                    }

                    IEnumerable<SBRElement> sbrDeclarationCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration", out currentValue))
                        sbrDeclarationCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationCollection = null;

                    #region sbrDeclarationCollection

                    if (sbrDeclarationCollection != null)
                    {

                        IEnumerable<SBRElement> sbrDeclarationCollection_TrueAndCorrect_RPCollection;
                        sbrDeclarationCollection_TrueAndCorrect_RPCollection = sbrDeclarationCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrDeclarationCollection_TrueAndCorrect_RPCollection != null && sbrDeclarationCollection_TrueAndCorrect_RPCollection.Count() > 0)
                        {
                            SBRElement sbrDeclarationCollection_TrueAndCorrect_RP;
                            sbrDeclarationCollection_TrueAndCorrect_RP = sbrDeclarationCollection_TrueAndCorrect_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap;
                            sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap = sbrDeclarationCollection_TrueAndCorrect_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.DeclarationCollection_TrueAndCorrect_RPExists = true;
                            report.DeclarationCollection_TrueAndCorrect_RPCount = sbrDeclarationCollection_TrueAndCorrect_RPCollection.Count();
                            report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_RP.OccurrenceIndex + 1;
                            report.DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR161 = currentValue.Value;
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementAccepted.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR162 = ToBoolean(currentValue.Value);
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Statement.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR122 = currentValue.Value;
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Signature.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR123 = DateTime.Parse(currentValue.Value);
                            }

                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.SignatoryIdentifier.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR160 = currentValue.Value;
                            }

                            IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection;
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName", out currentValue))
                                sbrDeclaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                            else
                                sbrDeclaration_PersonUnstructuredNameCollection = null;

                            #region sbrDeclaration_PersonUnstructuredNameCollection

                            if (sbrDeclaration_PersonUnstructuredNameCollection != null)
                            {

                                IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection;
                                sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection = sbrDeclaration_PersonUnstructuredNameCollection.Where(
                                    ce => ce.ChildElements != null &&
                                    ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                    sbrRPContexts.Contains(te.Context)));
                                if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection != null && sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count() > 0)
                                {
                                    SBRElement sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.ElementAt(0);

                                    IDictionary<string, SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count();
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.OccurrenceIndex + 1;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPLastOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Last().OccurrenceIndex + 1;

                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.Usage.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FITR166 = currentValue.Value;
                                    }

                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.FullName.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FITR167 = currentValue.Value;
                                    }
                                }
                            }
                            #endregion End of Tuple sbrDeclaration_PersonUnstructuredNameCollection
                            if (validationMode) ValidateRPDeclaration_PersonUnstructuredNameCollection(errors, sbrDeclaration_PersonUnstructuredNameCollection, sbrRPContexts, report);

                        }
                    }
                    #endregion End of Tuple sbrDeclarationCollection
                    if (validationMode) ValidateRPDeclarationCollection(errors, sbrDeclarationCollection, sbrRPContexts, report);


                    IEnumerable<SBRElement> sbrPersonNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails", out currentValue))
                        sbrPersonNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrPersonNameDetailsCollection = null;

                    #region sbrPersonNameDetailsCollection

                    if (sbrPersonNameDetailsCollection != null)
                    {

                        IEnumerable<SBRElement> sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection;
                        sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection = sbrPersonNameDetailsCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Contact") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection != null && sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrPersonNameDetailsCollection_LGL_Contact_C_RP;
                            sbrPersonNameDetailsCollection_LGL_Contact_C_RP = sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap;
                            sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap = sbrPersonNameDetailsCollection_LGL_Contact_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.PersonNameDetailsCollection_LGL_Contact_C_RPExists = true;
                            report.PersonNameDetailsCollection_LGL_Contact_C_RPCount = sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count();
                            report.PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_Contact_C_RP.OccurrenceIndex + 1;
                            report.PersonNameDetailsCollection_LGL_Contact_C_RPLastOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_Contact_C_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.PersonNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR138 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR140 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR139 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Title.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR102 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.NameSuffix.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR104 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.FamilyName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR103 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.GivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR105 = currentValue.Value;
                            }

                            if (sbrPersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.OtherGivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR106 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrPersonNameDetailsCollection
                    if (validationMode) ValidateRPPersonNameDetailsCollection(errors, sbrPersonNameDetailsCollection, sbrRPContexts, report);


                    IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContactTelephone", out currentValue))
                        sbrElectronicContactTelephoneCollection = currentValue.Occurrences;
                    else
                        sbrElectronicContactTelephoneCollection = null;

                    #region sbrElectronicContactTelephoneCollection

                    if (sbrElectronicContactTelephoneCollection != null)
                    {

                        IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection_03_RPCollection;
                        sbrElectronicContactTelephoneCollection_03_RPCollection = sbrElectronicContactTelephoneCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "ElectronicContact.Telephone.Usage.Code" && te.Value == "03" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrElectronicContactTelephoneCollection_03_RPCollection != null && sbrElectronicContactTelephoneCollection_03_RPCollection.Count() > 0)
                        {
                            SBRElement sbrElectronicContactTelephoneCollection_03_RP;
                            sbrElectronicContactTelephoneCollection_03_RP = sbrElectronicContactTelephoneCollection_03_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrElectronicContactTelephoneCollection_03_RPTupleElementMap;
                            sbrElectronicContactTelephoneCollection_03_RPTupleElementMap = sbrElectronicContactTelephoneCollection_03_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.ElectronicContactTelephoneCollection_03_RPExists = true;
                            report.ElectronicContactTelephoneCollection_03_RPCount = sbrElectronicContactTelephoneCollection_03_RPCollection.Count();
                            report.ElectronicContactTelephoneCollection_03_RPOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_RP.OccurrenceIndex + 1;
                            report.ElectronicContactTelephoneCollection_03_RPLastOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrElectronicContactTelephoneCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR141 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.ServiceLine.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR142 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Area.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR107 = currentValue.Value;
                            }

                            if (sbrElectronicContactTelephoneCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Minimal.Number", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR108 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrElectronicContactTelephoneCollection
                    if (validationMode) ValidateRPElectronicContactTelephoneCollection(errors, sbrElectronicContactTelephoneCollection, sbrRPContexts, report);


                    IEnumerable<SBRElement> sbrElectronicContactElectronicMailCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContactElectronicMail", out currentValue))
                        sbrElectronicContactElectronicMailCollection = currentValue.Occurrences;
                    else
                        sbrElectronicContactElectronicMailCollection = null;

                    #region sbrElectronicContactElectronicMailCollection

                    if (sbrElectronicContactElectronicMailCollection != null)
                    {
                        report.ElectronicContactElectronicMailCollectionExists = true;
                        report.ElectronicContactElectronicMailCollectionCount = sbrElectronicContactElectronicMailCollection.Count();

                        IEnumerable<SBRElement> sbrElectronicContactElectronicMailCollection_03_RPCollection;
                        sbrElectronicContactElectronicMailCollection_03_RPCollection = sbrElectronicContactElectronicMailCollection.Where(
                            ce => ce.ChildElements != null &&
                            ce.ChildElements.Any(te => te.Name == "ElectronicContact.ElectronicMail.Usage.Code" && te.Value == "03" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrElectronicContactElectronicMailCollection_03_RPCollection != null && sbrElectronicContactElectronicMailCollection_03_RPCollection.Count() > 0)
                        {
                            SBRElement sbrElectronicContactElectronicMailCollection_03_RP;
                            sbrElectronicContactElectronicMailCollection_03_RP = sbrElectronicContactElectronicMailCollection_03_RPCollection.ElementAt(0);

                            IDictionary<string, SBRElement> sbrElectronicContactElectronicMailCollection_03_RPTupleElementMap;
                            sbrElectronicContactElectronicMailCollection_03_RPTupleElementMap = sbrElectronicContactElectronicMailCollection_03_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

                            report.ElectronicContactElectronicMailCollection_03_RPExists = true;
                            report.ElectronicContactElectronicMailCollection_03_RPCount = sbrElectronicContactElectronicMailCollection_03_RPCollection.Count();
                            report.ElectronicContactElectronicMailCollection_03_RPOccurrenceIndex = sbrElectronicContactElectronicMailCollection_03_RP.OccurrenceIndex + 1;
                            report.ElectronicContactElectronicMailCollection_03_RPLastOccurrenceIndex = sbrElectronicContactElectronicMailCollection_03_RPCollection.Last().OccurrenceIndex + 1;

                            if (sbrElectronicContactElectronicMailCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.ElectronicMail.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR158 = currentValue.Value;
                            }

                            if (sbrElectronicContactElectronicMailCollection_03_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.ElectronicMail.Address.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FITR109 = currentValue.Value;
                            }
                        }
                    }
                    #endregion End of Tuple sbrElectronicContactElectronicMailCollection
                    if (validationMode) ValidateRPElectronicContactElectronicMailCollection(errors, sbrElectronicContactElectronicMailCollection, sbrRPContexts, report);


                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Report.CompletionHours.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR110 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR202 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPContext

            #region sbrRPInstantContext
            IEnumerable<SBRContext> sbrRPInstantContexts = null;
            SBRContext sbrRPInstantContext = null;

            sbrRPInstantContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.INSTANT &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInstantElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInstantContexts != null && sbrRPInstantContexts.Count() > 0)
            {
                sbrRPInstantContext = sbrRPInstantContexts.First();

                report.RPInstantId = sbrRPInstantContext.Id;
                report.RPInstantIdentifier = sbrRPInstantContext.EntityIdentifier;
                report.RPInstantIdentifierScheme = sbrRPInstantContext.EntityScheme;

                report.RPInstantCount = sbrRPInstantContexts.Count();
                report.RPInstantExists = true;
                report.RPInstantOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInstantContext) + 1;
                report.RPInstantLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInstantContexts.Last()) + 1;


                if (report.RPInstantIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInstantIdentifierTFN = report.RPInstantIdentifier;

                report.RPInstantInstantDate = ((SBRInstantPeriod)sbrRPInstantContext.Period).InstantAsDateObject;

                sbrPopulatedRPInstantElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInstantContexts.Count(); i++)
                {
                    sbrRPInstantContext = sbrRPInstantContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInstantContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInstantElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInstantElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInstantElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationControl.RegisteredMember.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR208 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPInstantContext

            #region sbrRPJAUSContext
            IEnumerable<SBRContext> sbrRPJAUSContexts = null;
            SBRContext sbrRPJAUSContext = null;

            sbrRPJAUSContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalJurisdictionDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Australian"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPJAUSElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPJAUSContexts != null && sbrRPJAUSContexts.Count() > 0)
            {
                sbrRPJAUSContext = sbrRPJAUSContexts.First();

                report.RPJAUSId = sbrRPJAUSContext.Id;
                report.RPJAUSIdentifier = sbrRPJAUSContext.EntityIdentifier;
                report.RPJAUSIdentifierScheme = sbrRPJAUSContext.EntityScheme;

                report.RPJAUSCount = sbrRPJAUSContexts.Count();
                report.RPJAUSExists = true;
                report.RPJAUSOccurrenceIndex = Array.IndexOf(contextArray, sbrRPJAUSContext) + 1;
                report.RPJAUSLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPJAUSContexts.Last()) + 1;


                if (report.RPJAUSIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPJAUSIdentifierTFN = report.RPJAUSIdentifier;

                report.RPJAUSStartDate = ((SBRDurationPeriod)sbrRPJAUSContext.Period).DurationStartAsDateObject;
                report.RPJAUSEndDate = ((SBRDurationPeriod)sbrRPJAUSContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPJAUSElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPJAUSContexts.Count(); i++)
                {
                    sbrRPJAUSContext = sbrRPJAUSContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPJAUSContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPJAUSElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPJAUSElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPJAUSElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPJAUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.Interest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR52 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPJAUSContext

            #region sbrRPJFOREIGNContext
            IEnumerable<SBRContext> sbrRPJFOREIGNContexts = null;
            SBRContext sbrRPJFOREIGNContext = null;

            sbrRPJFOREIGNContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalJurisdictionDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Foreign"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPJFOREIGNElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPJFOREIGNContexts != null && sbrRPJFOREIGNContexts.Count() > 0)
            {
                sbrRPJFOREIGNContext = sbrRPJFOREIGNContexts.First();

                report.RPJFOREIGNId = sbrRPJFOREIGNContext.Id;
                report.RPJFOREIGNIdentifier = sbrRPJFOREIGNContext.EntityIdentifier;
                report.RPJFOREIGNIdentifierScheme = sbrRPJFOREIGNContext.EntityScheme;

                report.RPJFOREIGNCount = sbrRPJFOREIGNContexts.Count();
                report.RPJFOREIGNExists = true;
                report.RPJFOREIGNOccurrenceIndex = Array.IndexOf(contextArray, sbrRPJFOREIGNContext) + 1;
                report.RPJFOREIGNLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPJFOREIGNContexts.Last()) + 1;


                if (report.RPJFOREIGNIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPJFOREIGNIdentifierTFN = report.RPJFOREIGNIdentifier;

                report.RPJFOREIGNStartDate = ((SBRDurationPeriod)sbrRPJFOREIGNContext.Period).DurationStartAsDateObject;
                report.RPJFOREIGNEndDate = ((SBRDurationPeriod)sbrRPJFOREIGNContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPJFOREIGNElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPJFOREIGNContexts.Count(); i++)
                {
                    sbrRPJFOREIGNContext = sbrRPJFOREIGNContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPJFOREIGNContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPJFOREIGNElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPJFOREIGNElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPJFOREIGNElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPJFOREIGNElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Gross.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR27 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPJFOREIGNElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.Net.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR28 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPJFOREIGNElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.Interest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR53 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPJFOREIGNContext

            #region sbrRPTOFAContext
            IEnumerable<SBRContext> sbrRPTOFAContexts = null;
            SBRContext sbrRPTOFAContext = null;

            sbrRPTOFAContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}FinancialArrangementTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TOFA"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTOFAElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTOFAContexts != null && sbrRPTOFAContexts.Count() > 0)
            {
                sbrRPTOFAContext = sbrRPTOFAContexts.First();

                report.RPTOFAId = sbrRPTOFAContext.Id;
                report.RPTOFAIdentifier = sbrRPTOFAContext.EntityIdentifier;
                report.RPTOFAIdentifierScheme = sbrRPTOFAContext.EntityScheme;

                report.RPTOFACount = sbrRPTOFAContexts.Count();
                report.RPTOFAExists = true;
                report.RPTOFAOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTOFAContext) + 1;
                report.RPTOFALastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTOFAContexts.Last()) + 1;


                if (report.RPTOFAIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTOFAIdentifierTFN = report.RPTOFAIdentifier;

                report.RPTOFAStartDate = ((SBRDurationPeriod)sbrRPTOFAContext.Period).DurationStartAsDateObject;
                report.RPTOFAEndDate = ((SBRDurationPeriod)sbrRPTOFAContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPTOFAElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTOFAContexts.Count(); i++)
                {
                    sbrRPTOFAContext = sbrRPTOFAContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTOFAContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTOFAElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTOFAElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTOFAElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPTOFAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.GainsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR134 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPTOFAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Expense.LossesTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR135 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPTOFAContext

            #region sbrRPClosingContext
            IEnumerable<SBRContext> sbrRPClosingContexts = null;
            SBRContext sbrRPClosingContext = null;

            sbrRPClosingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}OpeningAndClosingBalanceDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Closing"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPClosingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPClosingContexts != null && sbrRPClosingContexts.Count() > 0)
            {
                sbrRPClosingContext = sbrRPClosingContexts.First();

                report.RPClosingId = sbrRPClosingContext.Id;
                report.RPClosingIdentifier = sbrRPClosingContext.EntityIdentifier;
                report.RPClosingIdentifierScheme = sbrRPClosingContext.EntityScheme;

                report.RPClosingCount = sbrRPClosingContexts.Count();
                report.RPClosingExists = true;
                report.RPClosingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPClosingContext) + 1;
                report.RPClosingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPClosingContexts.Last()) + 1;


                if (report.RPClosingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPClosingIdentifierTFN = report.RPClosingIdentifier;

                report.RPClosingStartDate = ((SBRDurationPeriod)sbrRPClosingContext.Period).DurationStartAsDateObject;
                report.RPClosingEndDate = ((SBRDurationPeriod)sbrRPClosingContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPClosingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPClosingContexts.Count(); i++)
                {
                    sbrRPClosingContext = sbrRPClosingContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPClosingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPClosingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPClosingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPClosingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPClosingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR76 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPClosingContext

            #region sbrRPClosingInstantContext
            IEnumerable<SBRContext> sbrRPClosingInstantContexts = null;
            SBRContext sbrRPClosingInstantContext = null;

            sbrRPClosingInstantContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.INSTANT &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}OpeningAndClosingBalanceDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Closing"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPClosingInstantElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPClosingInstantContexts != null && sbrRPClosingInstantContexts.Count() > 0)
            {
                sbrRPClosingInstantContext = sbrRPClosingInstantContexts.First();

                report.RPClosingInstantId = sbrRPClosingInstantContext.Id;
                report.RPClosingInstantIdentifier = sbrRPClosingInstantContext.EntityIdentifier;
                report.RPClosingInstantIdentifierScheme = sbrRPClosingInstantContext.EntityScheme;

                report.RPClosingInstantCount = sbrRPClosingInstantContexts.Count();
                report.RPClosingInstantExists = true;
                report.RPClosingInstantOccurrenceIndex = Array.IndexOf(contextArray, sbrRPClosingInstantContext) + 1;
                report.RPClosingInstantLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPClosingInstantContexts.Last()) + 1;


                if (report.RPClosingInstantIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPClosingInstantIdentifierTFN = report.RPClosingInstantIdentifier;

                report.RPClosingInstantInstantDate = ((SBRInstantPeriod)sbrRPClosingInstantContext.Period).InstantAsDateObject;

                sbrPopulatedRPClosingInstantElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPClosingInstantContexts.Count(); i++)
                {
                    sbrRPClosingInstantContext = sbrRPClosingInstantContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPClosingInstantContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPClosingInstantElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPClosingInstantElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPClosingInstantElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPClosingInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Capital.Losses.CarriedForward.Net.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR207 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPClosingInstantContext

            #region sbrRPCFCLCContext
            IEnumerable<SBRContext> sbrRPCFCLCContexts = null;
            SBRContext sbrRPCFCLCContext = null;

            sbrRPCFCLCContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CountryOfResidencyCFCClassificationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ListedCountry"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCFCLCElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCFCLCContexts != null && sbrRPCFCLCContexts.Count() > 0)
            {
                sbrRPCFCLCContext = sbrRPCFCLCContexts.First();

                report.RPCFCLCId = sbrRPCFCLCContext.Id;
                report.RPCFCLCIdentifier = sbrRPCFCLCContext.EntityIdentifier;
                report.RPCFCLCIdentifierScheme = sbrRPCFCLCContext.EntityScheme;

                report.RPCFCLCCount = sbrRPCFCLCContexts.Count();
                report.RPCFCLCExists = true;
                report.RPCFCLCOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCLCContext) + 1;
                report.RPCFCLCLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCLCContexts.Last()) + 1;


                if (report.RPCFCLCIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCFCLCIdentifierTFN = report.RPCFCLCIdentifier;

                report.RPCFCLCStartDate = ((SBRDurationPeriod)sbrRPCFCLCContext.Period).DurationStartAsDateObject;
                report.RPCFCLCEndDate = ((SBRDurationPeriod)sbrRPCFCLCContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPCFCLCElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCFCLCContexts.Count(); i++)
                {
                    sbrRPCFCLCContext = sbrRPCFCLCContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCFCLCContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCFCLCElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCFCLCElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCFCLCElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.InternationalDealings.Attributed.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR78 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPCFCLCContext

            #region sbrRPCFCUCContext
            IEnumerable<SBRContext> sbrRPCFCUCContexts = null;
            SBRContext sbrRPCFCUCContext = null;

            sbrRPCFCUCContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CountryOfResidencyCFCClassificationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}UnlistedCountry"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCFCUCElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCFCUCContexts != null && sbrRPCFCUCContexts.Count() > 0)
            {
                sbrRPCFCUCContext = sbrRPCFCUCContexts.First();

                report.RPCFCUCId = sbrRPCFCUCContext.Id;
                report.RPCFCUCIdentifier = sbrRPCFCUCContext.EntityIdentifier;
                report.RPCFCUCIdentifierScheme = sbrRPCFCUCContext.EntityScheme;

                report.RPCFCUCCount = sbrRPCFCUCContexts.Count();
                report.RPCFCUCExists = true;
                report.RPCFCUCOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCUCContext) + 1;
                report.RPCFCUCLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCUCContexts.Last()) + 1;


                if (report.RPCFCUCIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCFCUCIdentifierTFN = report.RPCFCUCIdentifier;

                report.RPCFCUCStartDate = ((SBRDurationPeriod)sbrRPCFCUCContext.Period).DurationStartAsDateObject;
                report.RPCFCUCEndDate = ((SBRDurationPeriod)sbrRPCFCUCContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPCFCUCElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCFCUCContexts.Count(); i++)
                {
                    sbrRPCFCUCContext = sbrRPCFCUCContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCFCUCContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCFCUCElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCFCUCElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCFCUCElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Income.InternationalDealings.Attributed.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR80 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPCFCUCContext

            #region sbrRPLimitedPartnersContext
            IEnumerable<SBRContext> sbrRPLimitedPartnersContexts = null;
            SBRContext sbrRPLimitedPartnersContext = null;

            sbrRPLimitedPartnersContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}EarlyInvestorDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}LimitedPartners"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLimitedPartnersElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLimitedPartnersContexts != null && sbrRPLimitedPartnersContexts.Count() > 0)
            {
                sbrRPLimitedPartnersContext = sbrRPLimitedPartnersContexts.First();

                report.RPLimitedPartnersId = sbrRPLimitedPartnersContext.Id;
                report.RPLimitedPartnersIdentifier = sbrRPLimitedPartnersContext.EntityIdentifier;
                report.RPLimitedPartnersIdentifierScheme = sbrRPLimitedPartnersContext.EntityScheme;

                report.RPLimitedPartnersCount = sbrRPLimitedPartnersContexts.Count();
                report.RPLimitedPartnersExists = true;
                report.RPLimitedPartnersOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLimitedPartnersContext) + 1;
                report.RPLimitedPartnersLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLimitedPartnersContexts.Last()) + 1;


                if (report.RPLimitedPartnersIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLimitedPartnersIdentifierTFN = report.RPLimitedPartnersIdentifier;

                report.RPLimitedPartnersStartDate = ((SBRDurationPeriod)sbrRPLimitedPartnersContext.Period).DurationStartAsDateObject;
                report.RPLimitedPartnersEndDate = ((SBRDurationPeriod)sbrRPLimitedPartnersContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPLimitedPartnersElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLimitedPartnersContexts.Count(); i++)
                {
                    sbrRPLimitedPartnersContext = sbrRPLimitedPartnersContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLimitedPartnersContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLimitedPartnersElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLimitedPartnersElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLimitedPartnersElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPLimitedPartnersElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxOffsetClaim.NonRefundableOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR213 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPLimitedPartnersContext

            #region sbrRPInvestorContext
            IEnumerable<SBRContext> sbrRPInvestorContexts = null;
            SBRContext sbrRPInvestorContext = null;

            sbrRPInvestorContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}EarlyInvestorDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Investor"))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInvestorElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInvestorContexts != null && sbrRPInvestorContexts.Count() > 0)
            {
                sbrRPInvestorContext = sbrRPInvestorContexts.First();

                report.RPInvestorId = sbrRPInvestorContext.Id;
                report.RPInvestorIdentifier = sbrRPInvestorContext.EntityIdentifier;
                report.RPInvestorIdentifierScheme = sbrRPInvestorContext.EntityScheme;

                report.RPInvestorCount = sbrRPInvestorContexts.Count();
                report.RPInvestorExists = true;
                report.RPInvestorOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestorContext) + 1;
                report.RPInvestorLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestorContexts.Last()) + 1;


                if (report.RPInvestorIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInvestorIdentifierTFN = report.RPInvestorIdentifier;

                report.RPInvestorStartDate = ((SBRDurationPeriod)sbrRPInvestorContext.Period).DurationStartAsDateObject;
                report.RPInvestorEndDate = ((SBRDurationPeriod)sbrRPInvestorContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPInvestorElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInvestorContexts.Count(); i++)
                {
                    sbrRPInvestorContext = sbrRPInvestorContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInvestorContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInvestorElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInvestorElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInvestorElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPInvestorElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxOffsetClaim.NonRefundableOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR214 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPInvestorContext

            #region sbrRPInvestorYPreviousContext
            IEnumerable<SBRContext> sbrRPInvestorYPreviousContexts = null;
            SBRContext sbrRPInvestorYPreviousContext = null;

            sbrRPInvestorYPreviousContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}EarlyInvestorDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Investor")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}RelativePeriodDurationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}YPrevious")))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInvestorYPreviousElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInvestorYPreviousContexts != null && sbrRPInvestorYPreviousContexts.Count() > 0)
            {
                sbrRPInvestorYPreviousContext = sbrRPInvestorYPreviousContexts.First();

                report.RPInvestorYPreviousId = sbrRPInvestorYPreviousContext.Id;
                report.RPInvestorYPreviousIdentifier = sbrRPInvestorYPreviousContext.EntityIdentifier;
                report.RPInvestorYPreviousIdentifierScheme = sbrRPInvestorYPreviousContext.EntityScheme;

                report.RPInvestorYPreviousCount = sbrRPInvestorYPreviousContexts.Count();
                report.RPInvestorYPreviousExists = true;
                report.RPInvestorYPreviousOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestorYPreviousContext) + 1;
                report.RPInvestorYPreviousLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestorYPreviousContexts.Last()) + 1;


                if (report.RPInvestorYPreviousIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInvestorYPreviousIdentifierTFN = report.RPInvestorYPreviousIdentifier;

                report.RPInvestorYPreviousStartDate = ((SBRDurationPeriod)sbrRPInvestorYPreviousContext.Period).DurationStartAsDateObject;
                report.RPInvestorYPreviousEndDate = ((SBRDurationPeriod)sbrRPInvestorYPreviousContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPInvestorYPreviousElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInvestorYPreviousContexts.Count(); i++)
                {
                    sbrRPInvestorYPreviousContext = sbrRPInvestorYPreviousContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInvestorYPreviousContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInvestorYPreviousElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInvestorYPreviousElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInvestorYPreviousElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPInvestorYPreviousElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxOffsetClaim.NonRefundableOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR218 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPInvestorYPreviousContext

            #region sbrRPLimitedPartnersYPreviousContext
            IEnumerable<SBRContext> sbrRPLimitedPartnersYPreviousContexts = null;
            SBRContext sbrRPLimitedPartnersYPreviousContext = null;

            sbrRPLimitedPartnersYPreviousContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}EarlyInvestorDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}LimitedPartners")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}RelativePeriodDurationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}YPrevious")))))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLimitedPartnersYPreviousElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLimitedPartnersYPreviousContexts != null && sbrRPLimitedPartnersYPreviousContexts.Count() > 0)
            {
                sbrRPLimitedPartnersYPreviousContext = sbrRPLimitedPartnersYPreviousContexts.First();

                report.RPLimitedPartnersYPreviousId = sbrRPLimitedPartnersYPreviousContext.Id;
                report.RPLimitedPartnersYPreviousIdentifier = sbrRPLimitedPartnersYPreviousContext.EntityIdentifier;
                report.RPLimitedPartnersYPreviousIdentifierScheme = sbrRPLimitedPartnersYPreviousContext.EntityScheme;

                report.RPLimitedPartnersYPreviousCount = sbrRPLimitedPartnersYPreviousContexts.Count();
                report.RPLimitedPartnersYPreviousExists = true;
                report.RPLimitedPartnersYPreviousOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLimitedPartnersYPreviousContext) + 1;
                report.RPLimitedPartnersYPreviousLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLimitedPartnersYPreviousContexts.Last()) + 1;


                if (report.RPLimitedPartnersYPreviousIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLimitedPartnersYPreviousIdentifierTFN = report.RPLimitedPartnersYPreviousIdentifier;

                report.RPLimitedPartnersYPreviousStartDate = ((SBRDurationPeriod)sbrRPLimitedPartnersYPreviousContext.Period).DurationStartAsDateObject;
                report.RPLimitedPartnersYPreviousEndDate = ((SBRDurationPeriod)sbrRPLimitedPartnersYPreviousContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPLimitedPartnersYPreviousElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLimitedPartnersYPreviousContexts.Count(); i++)
                {
                    sbrRPLimitedPartnersYPreviousContext = sbrRPLimitedPartnersYPreviousContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLimitedPartnersYPreviousContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLimitedPartnersYPreviousElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLimitedPartnersYPreviousElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLimitedPartnersYPreviousElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPLimitedPartnersYPreviousElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}TaxOffsetClaim.NonRefundableOther.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FITR217 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPLimitedPartnersYPreviousContext
            return report;

        } // Of Consume Method
        #region ValidateINTDeclaration_PersonUnstructuredNameCollection
        protected virtual void ValidateINTDeclaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists = true;

                    SBRElement FITR168Value = null;
                    string FITR168 = null;
                    SBRElement FITR169Value = null;
                    string FITR169 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.Usage.Code", out FITR168Value))
                    {
                        if (!FITR168Value.IsNil) FITR168 = FITR168Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.FullName.Text", out FITR169Value))
                    {
                        if (!FITR169Value.IsNil) FITR169 = FITR169Value.Value;
                    }

                    #region VR.ATO.GEN.436279

                    /*  VR.ATO.GEN.436279
                        The unstructured person name tuple must include the Usage Code 'DeclarationSignatory'.

                    Legacy Rule Format:
                        WHERE IN TUPLE (personunstructuredname1.xx.xx:PersonUnstructuredName) IN TUPLE(declaration2.xx.xx:Declaration)
                        IF pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "DeclarationSignatory"
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^PersonUnstructuredName <> NULL) AND (^FITR168 <> 'DeclarationSignatory')

                    Data Elements:

                    ^FITR168 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']

                    ^PersonUnstructuredName = INT:Declaration:PersonUnstructuredName
                    */
                    assertion = (Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists != false && FITR168 != @"DeclarationSignatory");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.436279",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name must have a usage code of 'DeclarationSignatory'",
                            Location = FITR168Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.Usage.Code" : DeriveSbrElementLocation(FITR168Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.436279" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR168", Value = FITR168 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTDeclarationCollection
        protected virtual void ValidateINTDeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FITR164Value = null;
                    string FITR164 = null;
                    SBRElement FITR165Value = null;
                    bool? FITR165 = null;
                    SBRElement FITR124Value = null;
                    string FITR124 = null;
                    SBRElement FITR125Value = null;
                    DateTime? FITR125 = null;
                    SBRElement FITR163Value = null;
                    string FITR163 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementType.Code", out FITR164Value))
                    {
                        if (!FITR164Value.IsNil) FITR164 = FITR164Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementAccepted.Indicator", out FITR165Value))
                    {
                        if (!FITR165Value.IsNil) FITR165 = ToBoolean(FITR165Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Statement.Text", out FITR124Value))
                    {
                        if (!FITR124Value.IsNil) FITR124 = FITR124Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Signature.Date", out FITR125Value))
                    {
                        if (!FITR125Value.IsNil) FITR125 = DateTime.Parse(FITR125Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.SignatoryIdentifier.Text", out FITR163Value))
                    {
                        if (!FITR163Value.IsNil) FITR163 = FITR163Value.Value;
                    }

                    #region VR.ATO.GEN.438061

                    /*  VR.ATO.GEN.438061
                        If an Intermediary is present, Intermediary declaration statement type code must be supplied and must be “TrueAndCorrect”.

                    Legacy Rule Format:
                        IF INT<>NULL AND INT:pyin.xx.xx:Declaration.StatementType.Code <> "TrueAndCorrect"
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        ^INT <> NULL AND ^FITR164 <> 'TrueAndCorrect'

                    Data Elements:

                    ^FITR164 = INT:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']

                    ^INT = INT
                    */
                    assertion = (report.INTExists != false && FITR164 != @"TrueAndCorrect");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438061",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration statement type code is required.",
                            LongDescription = @"If an Intermediary is present, Intermediary declaration statement type code must be supplied and must be ""TrueAndCorrect"".",
                            Location = FITR164Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:Declaration.StatementType.Code" : DeriveSbrElementLocation(FITR164Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438061" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR164", Value = FITR164 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTPersonNameDetailsCollection
        protected virtual void ValidateINTPersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool PersonNameDetailsCollection_LGL_Contact_C_INTExists = true;

                    SBRElement FITR153Value = null;
                    string FITR153 = null;

                    SBRElement FITR154Value = null;
                    string FITR154 = null;

                    SBRElement FITR155Value = null;
                    string FITR155 = null;
                    SBRElement FITR111Value = null;
                    string FITR111 = null;
                    SBRElement FITR113Value = null;
                    string FITR113 = null;
                    SBRElement FITR112Value = null;
                    string FITR112 = null;
                    SBRElement FITR114Value = null;
                    string FITR114 = null;
                    SBRElement FITR115Value = null;
                    string FITR115 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.PersonNameType.Code", out FITR153Value))
                    {
                        if (!FITR153Value.IsNil) FITR153 = FITR153Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Currency.Code", out FITR154Value))
                    {
                        if (!FITR154Value.IsNil) FITR154 = FITR154Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Usage.Code", out FITR155Value))
                    {
                        if (!FITR155Value.IsNil) FITR155 = FITR155Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Title.Text", out FITR111Value))
                    {
                        if (!FITR111Value.IsNil) FITR111 = FITR111Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.NameSuffix.Text", out FITR113Value))
                    {
                        if (!FITR113Value.IsNil) FITR113 = FITR113Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.FamilyName.Text", out FITR112Value))
                    {
                        if (!FITR112Value.IsNil) FITR112 = FITR112Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.GivenName.Text", out FITR114Value))
                    {
                        if (!FITR114Value.IsNil) FITR114 = FITR114Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.OtherGivenName.Text", out FITR115Value))
                    {
                        if (!FITR115Value.IsNil) FITR115 = FITR115Value.Value;
                    }

                    #region VR.ATO.GEN.432435

                    /*  VR.ATO.GEN.432435
                        The name of the person to whom notices should be sent must have the Name Type code 'LGL' (legal), Currency Code 'C' (current) and Usage code 'MailRecipient'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                        IF (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> "LGL") OR (pyde.xx.xx:PersonNameDetails.Currency.Code <> "C") OR (pyde.xx.xx:PersonNameDetails.Usage.Code <> "Contact")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^PersonNameDetails <> NULL) AND ((^FITR153 <> 'LGL') OR (^FITR154 <> 'C') OR (^FITR155 <> 'Contact'))

                    Data Elements:

                    ^FITR153 = INT:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^FITR154 = INT:PersonNameDetails:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^FITR155 = INT:PersonNameDetails:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^PersonNameDetails = INT:PersonNameDetails
                    */
                    assertion = (PersonNameDetailsCollection_LGL_Contact_C_INTExists != false && (FITR153 != @"LGL" || FITR154 != @"C" || FITR155 != @"Contact"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.432435",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name Type code, Currency code and Usage code must be present and correct",
                            LongDescription = @"Each person name must have the Name Type code 'LGL' (legal), Currency Code 'C' (current) and Usage code 'Contact'. Please contact your software vendor in order to rectify this error",
                            Location = FITR112Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.FamilyName.Text" : DeriveSbrElementLocation(FITR112Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432435" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR153", Value = FITR153 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR154", Value = FITR154 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR155", Value = FITR155 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTOrganisationNameDetailsCollection
        protected virtual void ValidateINTOrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool OrganisationNameDetailsCollection_MN_C_INTExists = true;

                    SBRElement FITR159Value = null;
                    string FITR159 = null;

                    SBRElement FITR198Value = null;
                    string FITR198 = null;
                    SBRElement FITR116Value = null;
                    string FITR116 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out FITR159Value))
                    {
                        if (!FITR159Value.IsNil) FITR159 = FITR159Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out FITR198Value))
                    {
                        if (!FITR198Value.IsNil) FITR198 = FITR198Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out FITR116Value))
                    {
                        if (!FITR116Value.IsNil) FITR116 = FITR116Value.Value;
                    }

                    #region VR.ATO.GEN.438006

                    /*  VR.ATO.GEN.438006
                        Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
                        IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (pyin.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^OrganisationNameDetails <> NULL) AND ((^FITR159 <> 'MN') OR (^FITR198 <> 'C'))

                    Data Elements:

                    ^FITR159 = INT:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^FITR198 = INT:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^OrganisationNameDetails = INT:OrganisationNameDetails
                    */
                    assertion = (OrganisationNameDetailsCollection_MN_C_INTExists != false && (FITR159 != @"MN" || FITR198 != @"C"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438006",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.",
                            Location = FITR116Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FITR116Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438006" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR159", Value = FITR159 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR198", Value = FITR198 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTElectronicContactTelephoneCollection
        protected virtual void ValidateINTElectronicContactTelephoneCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool ElectronicContactTelephoneCollection_03_INTExists = true;

                    SBRElement FITR156Value = null;
                    string FITR156 = null;

                    SBRElement FITR157Value = null;
                    string FITR157 = null;
                    SBRElement FITR117Value = null;
                    string FITR117 = null;
                    SBRElement FITR118Value = null;
                    string FITR118 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Usage.Code", out FITR156Value))
                    {
                        if (!FITR156Value.IsNil) FITR156 = FITR156Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.ServiceLine.Code", out FITR157Value))
                    {
                        if (!FITR157Value.IsNil) FITR157 = FITR157Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Area.Code", out FITR117Value))
                    {
                        if (!FITR117Value.IsNil) FITR117 = FITR117Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Minimal.Number", out FITR118Value))
                    {
                        if (!FITR118Value.IsNil) FITR118 = FITR118Value.Value;
                    }

                    #region VR.ATO.GEN.500131

                    /*  VR.ATO.GEN.500131
                        Telephone must include usage code '03' and service line code '01' or '02'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
                        IF pyde.xx.xx:ElectronicContact.Telephone.Usage.Code <> "03" OR pyde.xx.xx:ElectronicContact.Telephone.ServiceLine.Code <> SET ("01", "02")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^ElectronicContactTelephone <> NULL) AND (^FITR156 <> '03' OR NotInSet(^FITR157, '"01", "02"'))

                    Data Elements:

                    ^FITR156 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Usage.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

                    ^FITR157 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

                    ^ElectronicContactTelephone = INT:ElectronicContactTelephone
                    */
                    assertion = (ElectronicContactTelephoneCollection_03_INTExists != false && (FITR156 != @"03" || !(IsMatch(FITR157, @"^(01|02)$", RegexOptions.IgnoreCase))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Telephone must include usage code '03' and service line code '01' or '02'.",
                            Location = FITR118Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:ElectronicContact.Telephone.Minimal.Number" : DeriveSbrElementLocation(FITR118Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR156", Value = FITR156 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR157", Value = FITR157 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPOrganisationNameDetailsCollection
        protected virtual void ValidateRPOrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPAddressDetailsCollection
        protected virtual void ValidateRPAddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FITR145Value = null;
                    string FITR145 = null;
                    bool AddressDetailsCollection_POS_C_RPExists = true;

                    SBRElement FITR144Value = null;
                    string FITR144 = null;
                    SBRElement FITR143Value = null;
                    bool? FITR143 = null;
                    SBRElement FITR8Value = null;
                    string FITR8 = null;
                    SBRElement FITR146Value = null;
                    string FITR146 = null;
                    SBRElement FITR147Value = null;
                    string FITR147 = null;
                    SBRElement FITR148Value = null;
                    string FITR148 = null;
                    SBRElement FITR9Value = null;
                    string FITR9 = null;
                    SBRElement FITR11Value = null;
                    string FITR11 = null;
                    SBRElement FITR10Value = null;
                    string FITR10 = null;
                    SBRElement FITR149Value = null;
                    string FITR149 = null;
                    SBRElement FITR150Value = null;
                    string FITR150 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Currency.Code", out FITR145Value))
                    {
                        if (!FITR145Value.IsNil) FITR145 = FITR145Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Usage.Code", out FITR144Value))
                    {
                        if (!FITR144Value.IsNil) FITR144 = FITR144Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.OverseasAddress.Indicator", out FITR143Value))
                    {
                        if (!FITR143Value.IsNil) FITR143 = ToBoolean(FITR143Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line1.Text", out FITR8Value))
                    {
                        if (!FITR8Value.IsNil) FITR8 = FITR8Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line2.Text", out FITR146Value))
                    {
                        if (!FITR146Value.IsNil) FITR146 = FITR146Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line3.Text", out FITR147Value))
                    {
                        if (!FITR147Value.IsNil) FITR147 = FITR147Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Line4.Text", out FITR148Value))
                    {
                        if (!FITR148Value.IsNil) FITR148 = FITR148Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.LocalityName.Text", out FITR9Value))
                    {
                        if (!FITR9Value.IsNil) FITR9 = FITR9Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Postcode.Text", out FITR11Value))
                    {
                        if (!FITR11Value.IsNil) FITR11 = FITR11Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.StateOrTerritory.Code", out FITR10Value))
                    {
                        if (!FITR10Value.IsNil) FITR10 = FITR10Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.CountryName.Text", out FITR149Value))
                    {
                        if (!FITR149Value.IsNil) FITR149 = FITR149Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}AddressDetails.Country.Code", out FITR150Value))
                    {
                        if (!FITR150Value.IsNil) FITR150 = FITR150Value.Value;
                    }

                    #region VR.ATO.GEN.428202

                    /*  VR.ATO.GEN.428202
                        Address currency code must be "C" or "P"

                    Legacy Rule Format:
                        IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK)
                        AND (pyde.xx.xx:AddressDetails.Currency.Code <> SET("C","P"))
                         RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^FITR145 <> BLANK) AND (NotInSet(^FITR145, '"C","P"'))

                    Data Elements:

                    ^FITR145 = RP:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(FITR145) != true && !(IsMatch(FITR145, @"^(C|P)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428202",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address currency code must be ""C"" or ""P""",
                            Location = FITR145Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Currency.Code" : DeriveSbrElementLocation(FITR145Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428202" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR145", Value = FITR145 });

                        errors.Add(processMessage);
                    }
                    #endregion

                    #region VR.ATO.GEN.432434

                    /*  VR.ATO.GEN.432434
                        Address usage code must be 'POS' and Currency Code must be 'C'

                    Legacy Rule Format:
                        WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                        IF (pyde.xx.xx:AddressDetails.Usage.Code <> "POS") OR (pyde.xx.xx:AddressDetails.Currency.Code <> "C") 
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^AddressDetails <> NULL) AND ((^FITR144 <> 'POS') OR (^FITR145 <> 'C'))

                    Data Elements:

                    ^FITR144 = RP:AddressDetails:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

                    ^FITR145 = RP:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

                    ^AddressDetails = RP:AddressDetails
                    */
                    assertion = (AddressDetailsCollection_POS_C_RPExists != false && (FITR144 != @"POS" || FITR145 != @"C"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.432434",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address usage code must be 'POS' and Currency Code must be 'C'",
                            LongDescription = @"An address must have a Usage code 'POS' (postal) and a Currency Code 'C' (current). Please contact your software vendor in order to rectify this error",
                            Location = FITR8Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Line1.Text" : DeriveSbrElementLocation(FITR8Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432434" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR144", Value = FITR144 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR145", Value = FITR145 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTrustee_OrganisationNameDetailsCollection
        protected virtual void ValidateRPTrustee_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool Trustee_OrganisationNameDetailsCollection_MN_C_RPExists = true;

                    SBRElement FITR152Value = null;
                    string FITR152 = null;

                    SBRElement FITR194Value = null;
                    string FITR194 = null;
                    SBRElement FITR12Value = null;
                    string FITR12 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out FITR152Value))
                    {
                        if (!FITR152Value.IsNil) FITR152 = FITR152Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out FITR194Value))
                    {
                        if (!FITR194Value.IsNil) FITR194 = FITR194Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out FITR12Value))
                    {
                        if (!FITR12Value.IsNil) FITR12 = FITR12Value.Value;
                    }

                    #region VR.ATO.GEN.438006

                    /*  VR.ATO.GEN.438006
                        Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
                        IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (pyin.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^OrganisationNameDetails <> NULL) AND ((^FITR152 <> 'MN') OR (^FITR194 <> 'C'))

                    Data Elements:

                    ^FITR152 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^FITR194 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^OrganisationNameDetails = RP:Trustee:OrganisationNameDetails
                    */
                    assertion = (Trustee_OrganisationNameDetailsCollection_MN_C_RPExists != false && (FITR152 != @"MN" || FITR194 != @"C"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438006",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.",
                            Location = FITR12Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FITR12Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438006" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR152", Value = FITR152 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR194", Value = FITR194 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTransferee_OrganisationNameDetailsCollection
        protected virtual void ValidateRPTransferee_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool Transferee_OrganisationNameDetailsCollection_MN_C_RPExists = true;

                    SBRElement FITR170Value = null;
                    string FITR170 = null;

                    SBRElement FITR196Value = null;
                    string FITR196 = null;
                    SBRElement FITR85Value = null;
                    string FITR85 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalNameType.Code", out FITR170Value))
                    {
                        if (!FITR170Value.IsNil) FITR170 = FITR170Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.Currency.Code", out FITR196Value))
                    {
                        if (!FITR196Value.IsNil) FITR196 = FITR196Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}OrganisationNameDetails.OrganisationalName.Text", out FITR85Value))
                    {
                        if (!FITR85Value.IsNil) FITR85 = FITR85Value.Value;
                    }

                    #region VR.ATO.GEN.438006

                    /*  VR.ATO.GEN.438006
                        Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
                        IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (pyin.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^OrganisationNameDetails <> NULL) AND ((^FITR170 <> 'MN') OR (^FITR196 <> 'C'))

                    Data Elements:

                    ^FITR170 = RP:Transferee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^FITR196 = RP:Transferee:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                    ^OrganisationNameDetails = RP:Transferee:OrganisationNameDetails
                    */
                    assertion = (Transferee_OrganisationNameDetailsCollection_MN_C_RPExists != false && (FITR170 != @"MN" || FITR196 != @"C"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438006",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.",
                            Location = FITR85Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FITR85Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438006" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR170", Value = FITR170 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR196", Value = FITR196 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclaration_PersonUnstructuredNameCollection
        protected virtual void ValidateRPDeclaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;

                    SBRElement FITR166Value = null;
                    string FITR166 = null;
                    SBRElement FITR167Value = null;
                    string FITR167 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.Usage.Code", out FITR166Value))
                    {
                        if (!FITR166Value.IsNil) FITR166 = FITR166Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonUnstructuredName.FullName.Text", out FITR167Value))
                    {
                        if (!FITR167Value.IsNil) FITR167 = FITR167Value.Value;
                    }

                    #region VR.ATO.GEN.436279

                    /*  VR.ATO.GEN.436279
                        The unstructured person name tuple must include the Usage Code 'DeclarationSignatory'.

                    Legacy Rule Format:
                        WHERE IN TUPLE (personunstructuredname1.xx.xx:PersonUnstructuredName) IN TUPLE(declaration2.xx.xx:Declaration)
                        IF pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "DeclarationSignatory"
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^PersonUnstructuredName <> NULL) AND (^FITR166 <> 'DeclarationSignatory')

                    Data Elements:

                    ^FITR166 = RP:Declaration:PersonUnstructuredName:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']

                    ^PersonUnstructuredName = RP:Declaration:PersonUnstructuredName
                    */
                    assertion = (Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists != false && FITR166 != @"DeclarationSignatory");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.436279",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name must have a usage code of 'DeclarationSignatory'",
                            Location = FITR166Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.Usage.Code" : DeriveSbrElementLocation(FITR166Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.436279" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR166", Value = FITR166 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationCollection
        protected virtual void ValidateRPDeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FITR161Value = null;
                    string FITR161 = null;
                    SBRElement FITR162Value = null;
                    bool? FITR162 = null;
                    SBRElement FITR122Value = null;
                    string FITR122 = null;
                    SBRElement FITR123Value = null;
                    DateTime? FITR123 = null;
                    SBRElement FITR160Value = null;
                    string FITR160 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementType.Code", out FITR161Value))
                    {
                        if (!FITR161Value.IsNil) FITR161 = FITR161Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.StatementAccepted.Indicator", out FITR162Value))
                    {
                        if (!FITR162Value.IsNil) FITR162 = ToBoolean(FITR162Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Statement.Text", out FITR122Value))
                    {
                        if (!FITR122Value.IsNil) FITR122 = FITR122Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.Signature.Date", out FITR123Value))
                    {
                        if (!FITR123Value.IsNil) FITR123 = DateTime.Parse(FITR123Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}Declaration.SignatoryIdentifier.Text", out FITR160Value))
                    {
                        if (!FITR160Value.IsNil) FITR160 = FITR160Value.Value;
                    }

                    #region VR.ATO.GEN.438062

                    /*  VR.ATO.GEN.438062
                        If an Intermediary is not present, Reporting Party Declaration Statement Type Code must be supplied and must be “TrueAndCorrect”.

                    Legacy Rule Format:
                        IF INT=NULL AND RP:pyin.xx.xx:Declaration.StatementType.Code <> "TrueAndCorrect"
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        ^INT = NULL AND ^FITR161 <> 'TrueAndCorrect'

                    Data Elements:

                    ^FITR161 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']

                    ^INT = INT
                    */
                    assertion = (report.INTExists == false && FITR161 != @"TrueAndCorrect");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438062",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration statement type code is required.",
                            LongDescription = @"If an Intermediary is not present, Reporting Party Declaration Statement Type Code must be supplied and must be ""TrueAndCorrect"".",
                            Location = FITR161Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:Declaration.StatementType.Code" : DeriveSbrElementLocation(FITR161Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438062" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR161", Value = FITR161 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPPersonNameDetailsCollection
        protected virtual void ValidateRPPersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool PersonNameDetailsCollection_LGL_Contact_C_RPExists = true;

                    SBRElement FITR138Value = null;
                    string FITR138 = null;

                    SBRElement FITR139Value = null;
                    string FITR139 = null;

                    SBRElement FITR140Value = null;
                    string FITR140 = null;
                    SBRElement FITR102Value = null;
                    string FITR102 = null;
                    SBRElement FITR104Value = null;
                    string FITR104 = null;
                    SBRElement FITR103Value = null;
                    string FITR103 = null;
                    SBRElement FITR105Value = null;
                    string FITR105 = null;
                    SBRElement FITR106Value = null;
                    string FITR106 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.PersonNameType.Code", out FITR138Value))
                    {
                        if (!FITR138Value.IsNil) FITR138 = FITR138Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Currency.Code", out FITR139Value))
                    {
                        if (!FITR139Value.IsNil) FITR139 = FITR139Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Usage.Code", out FITR140Value))
                    {
                        if (!FITR140Value.IsNil) FITR140 = FITR140Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.Title.Text", out FITR102Value))
                    {
                        if (!FITR102Value.IsNil) FITR102 = FITR102Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.NameSuffix.Text", out FITR104Value))
                    {
                        if (!FITR104Value.IsNil) FITR104 = FITR104Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.FamilyName.Text", out FITR103Value))
                    {
                        if (!FITR103Value.IsNil) FITR103 = FITR103Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.GivenName.Text", out FITR105Value))
                    {
                        if (!FITR105Value.IsNil) FITR105 = FITR105Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}PersonNameDetails.OtherGivenName.Text", out FITR106Value))
                    {
                        if (!FITR106Value.IsNil) FITR106 = FITR106Value.Value;
                    }

                    #region VR.ATO.GEN.432435

                    /*  VR.ATO.GEN.432435
                        The name of the person to whom notices should be sent must have the Name Type code 'LGL' (legal), Currency Code 'C' (current) and Usage code 'MailRecipient'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                        IF (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> "LGL") OR (pyde.xx.xx:PersonNameDetails.Currency.Code <> "C") OR (pyde.xx.xx:PersonNameDetails.Usage.Code <> "Contact")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^PersonNameDetails <> NULL) AND ((^FITR138 <> 'LGL') OR (^FITR139 <> 'C') OR (^FITR140 <> 'Contact'))

                    Data Elements:

                    ^FITR138 = RP:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^FITR139 = RP:PersonNameDetails:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^FITR140 = RP:PersonNameDetails:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

                    ^PersonNameDetails = RP:PersonNameDetails
                    */
                    assertion = (PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && (FITR138 != @"LGL" || FITR139 != @"C" || FITR140 != @"Contact"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.432435",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name Type code, Currency code and Usage code must be present and correct",
                            LongDescription = @"Each person name must have the Name Type code 'LGL' (legal), Currency Code 'C' (current) and Usage code 'Contact'. Please contact your software vendor in order to rectify this error",
                            Location = FITR103Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.FamilyName.Text" : DeriveSbrElementLocation(FITR103Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432435" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR138", Value = FITR138 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR139", Value = FITR139 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR140", Value = FITR140 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPElectronicContactTelephoneCollection
        protected virtual void ValidateRPElectronicContactTelephoneCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool ElectronicContactTelephoneCollection_03_RPExists = true;

                    SBRElement FITR141Value = null;
                    string FITR141 = null;

                    SBRElement FITR142Value = null;
                    string FITR142 = null;
                    SBRElement FITR107Value = null;
                    string FITR107 = null;
                    SBRElement FITR108Value = null;
                    string FITR108 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Usage.Code", out FITR141Value))
                    {
                        if (!FITR141Value.IsNil) FITR141 = FITR141Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.ServiceLine.Code", out FITR142Value))
                    {
                        if (!FITR142Value.IsNil) FITR142 = FITR142Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Area.Code", out FITR107Value))
                    {
                        if (!FITR107Value.IsNil) FITR107 = FITR107Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.Telephone.Minimal.Number", out FITR108Value))
                    {
                        if (!FITR108Value.IsNil) FITR108 = FITR108Value.Value;
                    }

                    #region VR.ATO.GEN.500131

                    /*  VR.ATO.GEN.500131
                        Telephone must include usage code '03' and service line code '01' or '02'.

                    Legacy Rule Format:
                        WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
                        IF pyde.xx.xx:ElectronicContact.Telephone.Usage.Code <> "03" OR pyde.xx.xx:ElectronicContact.Telephone.ServiceLine.Code <> SET ("01", "02")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^ElectronicContactTelephone <> NULL) AND (^FITR141 <> '03' OR NotInSet(^FITR142, '"01", "02"'))

                    Data Elements:

                    ^FITR141 = RP:ElectronicContactTelephone:ElectronicContact.Telephone.Usage.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

                    ^FITR142 = RP:ElectronicContactTelephone:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

                    ^ElectronicContactTelephone = RP:ElectronicContactTelephone
                    */
                    assertion = (ElectronicContactTelephoneCollection_03_RPExists != false && (FITR141 != @"03" || !(IsMatch(FITR142, @"^(01|02)$", RegexOptions.IgnoreCase))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Telephone must include usage code '03' and service line code '01' or '02'.",
                            Location = FITR108Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:ElectronicContact.Telephone.Minimal.Number" : DeriveSbrElementLocation(FITR108Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR141", Value = FITR141 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR142", Value = FITR142 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPElectronicContactElectronicMailCollection
        protected virtual void ValidateRPElectronicContactElectronicMailCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FITR2021 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                    if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool ElectronicContactElectronicMailCollection_03_RPExists = true;

                    SBRElement FITR109Value = null;
                    string FITR109 = null;

                    SBRElement FITR158Value = null;
                    string FITR158 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.ElectronicMail.Address.Text", out FITR109Value))
                    {
                        if (!FITR109Value.IsNil) FITR109 = FITR109Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fitr}ElectronicContact.ElectronicMail.Usage.Code", out FITR158Value))
                    {
                        if (!FITR158Value.IsNil) FITR158 = FITR158Value.Value;
                    }

                    #region VR.ATO.GEN.500140

                    /*  VR.ATO.GEN.500140
                        Email address must include an address and a usage code '03'

                    Legacy Rule Format:
                        WHERE IN TUPLE(ElectronicContactElectronicMail1.xx.xx:ElectronicContactElectronicMail)
                        IF (pyde.xx.xx:ElectronicContact.ElectronicMail.Address.Text = BLANK) OR (pyde.xx.xx:ElectronicContact.ElectronicMail.Usage.Code <> "03")
                            RETURN VALIDATION MESSAGE
                        ENDIF

                    Technical Business Rule Format:
                        (^ElectronicContactElectronicMail <> NULL) AND ((^FITR109 = BLANK) OR (^FITR158 <> '03'))

                    Data Elements:

                    ^FITR158 = RP:ElectronicContactElectronicMail:ElectronicContact.ElectronicMail.Usage.Code WHERE [ElectronicContact.ElectronicMail.Usage.Code='03']

                    ^FITR109 = RP:ElectronicContactElectronicMail:ElectronicContact.ElectronicMail.Address.Text WHERE [ElectronicContact.ElectronicMail.Usage.Code='03']

                    ^ElectronicContactElectronicMail = RP:ElectronicContactElectronicMail
                    */
                    assertion = (ElectronicContactElectronicMailCollection_03_RPExists != false && (string.IsNullOrWhiteSpace(FITR109) == true || FITR158 != @"03"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500140",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Email address must include an address and a usage code '03'.",
                            Location = FITR158Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:ElectronicContact.ElectronicMail.Usage.Code" : DeriveSbrElementLocation(FITR158Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500140" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR109", Value = FITR109 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR158", Value = FITR158 });

                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

    } // Of Class
} // Of Namespace

